


density.plots <- function(fn, fix.fn=NULL, ref.g="ENSG00000101558", srp55.de=NULL, correct.fc=NULL)
{
    tx <- fread.transcript(fn, ref.g=ref.g, gtf=gtf, mode="Gene", correct.fc=correct.fc)
    tx.nn <-  fread.transcript(fn, gtf=gtf, normalize=F)

    if (! is.null(fix.fn) )
    {
        tx <- fix.fn(tx)
        tx.nn <- fix.fn(tx.nn)
    }

    sample_ids <- colnames(tx)[6:ncol(tx)]
    split_sample_ids <- strsplit(sample_ids, "-")
    conditions <- unique(sapply(split_sample_ids, function (x) x[[1]]))
    treatment <- unique(sapply(split_sample_ids, function (x) paste0(x[[2]], "-", x[[3]]) ))
    replicates <- unique(sapply(split_sample_ids, function (x) x[[4]]))
   

    nn <- do.call(rbind,
            lapply(conditions,
                   function (condition)
                   {
                       do.call(rbind,
                               lapply(replicates,
                                      function (replicate)
                                      {
                                          col <- paste0(condition, "-", treatment, "-", replicate)
                                          norm <- paste0("Normalized-", condition)
                                          raw <- paste0("Raw-", condition)
                                          dt <- rbind(tx[,..col][,color:=norm][,group:=paste0("Norm-", col)][,norm:=TRUE],
                                                      tx.nn[,..col][,color:=raw][,group:=paste0("Raw-",col)][,norm:=FALSE])
                                          setnames(dt, col, "TPM")
                                          dt
                                      }))
                   }))
    
    nn$color  <- as.factor(nn$color)
    nn$group  <- as.factor(nn$group)
    ## nn$condition  <- as.factor(nn$condition)
    gg.whole <- ggpubr::ggarrange(ggplot(nn[norm == TRUE,], aes(x=log(TPM), group=group, color=color) ) + geom_density(),
                                  ggplot(nn[norm == FALSE,], aes(x=log(TPM), group=group, color=color) ) + geom_density(), nrow=2)
    
    gg.pp <- ggpubr::ggarrange(pair.plot(tx, "ENST00000244020") + ggtitle(paste0("SRSF6\n", "p = ", formatC(srp55.de$main.pvalue, digit=2), "\n",
                                                                                "lFC = ", formatC(srp55.de$main.log2FoldChange, digit=2))),
                               pair.plot(tx.nn, "ENST00000244020"), nrow=2)
    
    ggpubr::ggarrange(gg.whole, gg.pp, ncol=2, widths=c(5,1))
}

density.plots.endoc.cyt <- function (.shortid, de=NULL)
{
    all.refgenes <- gtf$data[shortid %in% .shortid & feature == "gene",]$ensg
    correct.fc <- if (! is.null(de) ) sapply(all.refgenes, function (ensg) de[Row.names == ensg, 2**log2FoldChange]) else NULL
    print(correct.fc)
    annotate_figure(density.plots("input/de-florian/02-DE_endoC-48h_CYT_vs_CTL/differential_expression/02-DE_endoC-48h_CYT_vs_CTL-ALL_TX_salmon.csv",
                                  fix.fn=fix.endoc.cyt.48h.colnames, ref.g=all.refgenes, srp55.de=srp55.de[4,],
                                  correct.fc = correct.fc ),
                    paste("Normalized by", paste(.shortid, collapse=", ")) )
    
}

not.deg <- function(all.refgenes.shortid, det, mode=c("TX", "Gene") )
{
    if ( "TX" == mode )
        det[det[gene_name %in% all.refgenes.shortid,.I[baseMean == max(baseMean)],by=gene_name]$V1,][pvalue > 0.05, gene_name]
    else if ( "Gene" == mode )
    {
        det[gene_name %in% all.refgenes.shortid & pvalue > 0.05, gene_name]
    }
    else
        stop("Mode should be 'TX' or 'Gene'.")
}


density.plots.endoc.IFNa <- function (.shortid, de=NULL)
{
    all.refgenes <- gtf$data[shortid %in% .shortid & feature == "gene",]$ensg
    correct.fc <- if (! is.null(de) ) sapply(all.refgenes, function (ensg) de[Row.names == ensg, 2**log2FoldChange]) else NULL
    
    annotate_figure(density.plots("input/de-florian/02-DE_endoC-24h_IFNa_vs_CTL/differential_expression/02-DE_endoC-24h_IFNa_vs_CTL-ALL_TX_salmon.csv",
                                  fix.fn=fix.endoc.IFNa.colnames, ref.g=all.refgenes, srp55.de=srp55.de[3,],
                                  correct.fc = correct.fc),
                    paste("Normalized by", paste(.shortid, collapse=", ")) )
}
