
library(data.table)

newGTF <- function (filename="/srv/genomic_data/ensembl/GRCh37/gencode.v23lift37.annotation.gtf.gz")
{
    gtf <- fread(filename, skip=5, header=FALSE,
                 col.names=c("chr", "source", "feature", "start", "end", "score", "strand", "frame", "attribute"))

    gtf[,chr:=gsub("chr", "", chr)]
    gtf[feature == "gene", ensg:=gsub('.*gene_id "([^".]+)(\\.\\d+)?".*', "\\1", attribute)]
    gtf[feature == "gene", shortid:=gsub('.*gene_name "([^"]+)".*', "\\1", attribute)]
    gtf[feature == "gene", biotype:=ifelse(grepl("gene_biotype", attribute),
					   gsub('.*gene_biotype "([^"]+)".*', "\\1", attribute),
					   gsub('.*gene_type "([^"]+)".*', "\\1", attribute)) ]

    gtf[feature == "transcript", enst:=gsub('.*transcript_id "([^".]+)(\\.\\d+)?".*', "\\1", attribute)]
    gtf[feature == "transcript", ensg:=gsub('.*gene_id "([^".]+)(\\.\\d+)?".*', "\\1", attribute)]
    gtf[feature == "transcript", shortid:=gsub('.*gene_name "([^"]+)".*', "\\1", attribute)]
    gtf[feature == "transcript", 
	biotype:=ifelse(grepl("transcript_biotype", attribute), 
			gsub('.*transcript_biotype "([^"]+)".*', "\\1", attribute),
			gsub('.*transcript_type "([^"]+)".*', "\\1", attribute)) ]

    gtf[feature == "transcript" | feature == "gene", 
	ensp:=ifelse(grepl("protein_id", attribute), 
		     gsub('.*protein_id "([^".]+)(\\.\\d+)?".*', "\\1", attribute),
		     NA) ]

    structure(list(
        data   = gtf
    ),
        class  = c("gtf") )
}

genes <- function (self, ...)
{
    UseMethod("genes", self)
}

genes.gtf <- function(gtf, genes)
{
    gtf.genes <- gtf$data[feature == "gene",]

    return(gtf.genes[match(genes, gtf.genes$ensg),])
}
    
