
library(RRHO)

defaultStepSize <-function(list1, list2)
{
    n1<- dim(list1)[1]
    n2<- dim(list2)[1]
    result <- ceiling(min(sqrt(c(n1,n2))))
    
    return(result)
}

originalRRHO <- function (list1, list2, stepsize = defaultStepSize(list1, list2), 
                                  labels = NULL, alternative = "two.sided", plots = FALSE, outputdir = NULL, BY = FALSE, 
                                  log10.ind = FALSE)
{
    if ( is.null(outputdir) )
    {
        outputdir <- tempfile()
        dir.create(outputdir, showWarnings = FALSE)
    }

    if ( is.null(labels) )
    {
        labels = c("A", "B")
    }

    ret <- RRHO::RRHO(list1, list2, stepsize = stepsize, labels = labels, alternative = alternative, plots = TRUE, outputdir = outputdir, BY = BY, log10.ind = log10.ind)
    ret$outputdir <- outputdir


    nlist1 <- length(list1[, 1])
    nlist2 <- length(list2[, 1])


    ## This is from the original R package
    hypermat.signed <- ret$hypermat * ret$hypermat.signs
    
    maxind.ur <- which(max(hypermat.signed[ceiling(nrow(hypermat.signed)/2):nrow(hypermat.signed), 
                                           ceiling(ncol(hypermat.signed)/2):ncol(hypermat.signed)], 
                           na.rm = TRUE) == hypermat.signed, arr.ind = TRUE)
    indlist1.ur <- seq(1, nlist1, stepsize)[maxind.ur[1]]
    indlist2.ur <- seq(1, nlist2, stepsize)[maxind.ur[2]]
    genelist.ur <- intersect(list1[indlist1.ur:nlist1, 
                                   1], list2[indlist2.ur:nlist2, 1])
    
    maxind.lr <- which(max(hypermat.signed[1:(ceiling(nrow(hypermat.signed)/2) - 
                                              1), 1:(ceiling(ncol(hypermat.signed)/2) - 1)], 
                           na.rm = TRUE) == hypermat.signed, arr.ind = TRUE)
    indlist1.lr <- seq(1, nlist1, stepsize)[maxind.lr[1]]
    indlist2.lr <- seq(1, nlist2, stepsize)[maxind.lr[2]]
    genelist.lr <- intersect(list1[1:indlist1.lr, 1], 
                             list2[1:indlist2.lr, 1])
    
    .filename <- paste(outputdir, "/RRHO_GO_MostDownregulated", 
                       labels[1], "_VS_", labels[2], ".csv", sep = "")
    
    ret$downdown <- list(i = indlist1.ur,
                         j = indlist2.ur,
                         intersection = fread(.filename)$V1)
    
    .filename <- paste(outputdir, "/RRHO_GO_MostUpregulated", 
                       labels[1], "_VS_", labels[2], ".csv", sep = "")
    ret$upup <- list(i = indlist1.lr,
                     j = indlist2.lr,
                     intersection = fread(.filename)$V1)
    ######################### end of original R package
   

    ret$stepsize  <- stepsize

    return(ret)
}

## A.df <- data.frame((1:5000) - 2500, (1:5000) - 2500)
## originalRRHO(A.df, A.df, outputdir = "/tmp/test")
