#! /usr/bin/env Rscript

library(RedRibbon)
library(data.table)
library(magrittr)
library(egg)

quad <- function ()
{
    n <- 16000
    nh <- n / 2
    ah <- (1:nh) - 4 * nh / 10
    rah <-  rev(ah)
    a <- c(ah, ah)
    b <- c(ah, rah)
    rr <-  RedRibbon(a, b) %>% setoptions(enrichment_mode = "hyper-two-tailed")
    quadrants <- quadrants(rr, whole=FALSE, permutation=TRUE, m=100, n=100)
    gg_quad <- ggplot(rr, quadrants=quadrants, base_size = 16) +
        coord_fixed(ratio=1, clip = "off")

    return(gg_quad)
}

random <- function (show.quadrants=TRUE, show.pval=TRUE)
{
    n <- 16000
    nh <- n / 2
    ## random
    rr <- RedRibbon(sample(n)-nh, sample(n)-nh) %>% setoptions(enrichment_mode = "hyper-two-tailed")
    quadrants <- quadrants(rr, whole=FALSE, permutation=TRUE, m=100, n=100)
    gg_random <- ggplot(rr, quadrants=quadrants, base_size = 16,
                        show.quadrants = show.quadrants, show.pval = show.pval) +
        coord_fixed(ratio=1, clip = "off")

    return(gg_random)
}

perfect <- function ()
{
    n <- 16000
    nh <- n / 2

##############  Figure 2

    ## rm <- theme(plot.margin=margin(-3, 0, -3, 0, "cm"))

    ## Perfect

    rr <- RedRibbon(1:n, 1:n) %>% setoptions(enrichment_mode = "hyper-two-tailed")
    gg_perfect <- ggplot(rr, base_size = 16, show.quadrants=FALSE) + coord_fixed(ratio=1, clip = "off") +
        theme(legend.position = c(0.9, 0.5),
              legend.background = element_rect(fill=alpha('white', 0.2)))

    ## Perfect reverted

    rr <- RedRibbon(1:n, n:1) %>% setoptions(enrichment_mode = "hyper-two-tailed")
    gg_perfect_rev <- ggplot(rr, base_size = 16, show.quadrants=FALSE) + coord_fixed(ratio=1, clip = "off") +
        theme(legend.position = c(0.9, 0.5),
              legend.background = element_rect(fill=alpha('white', 0.2)))

    ## random
    gg_random <- random() +
        theme(legend.position = c(0.9, 0.5),
              legend.background = element_rect(fill=alpha('white', 0.2)))

    ## quad
    gg_quad <- quad() +
        theme(legend.position = c(0.9, 0.5),
              legend.background = element_rect(fill=alpha('white', 0.2)))


    figure_2 <- ggpubr::ggarrange(gg_perfect, gg_perfect_rev, gg_quad, gg_random, 
                                  labels = c("A", "B", "C", "D"),
                                  ncol = 2, nrow = 2) + bgcolor("#FFFFFF") +
        theme(panel.background = element_rect(color = "white", size = 0, 
                                              linetype = NULL),
              axis.line = element_blank())

    return(figure_2)
}
