library(ggplot2)

performance <- function(redribbon.fn="input/p-rrho.tsv", rrho.original.fn = "input/p-rrho-original.tsv")
{
    title.size <- 12
    text.size <- 16
    x.angle <- 25
    p.rrho <- fread(redribbon.fn, sep="\t")
    p.rrho.orig <- fread(rrho.original.fn, sep="\t")
    
    gg_rrho <- ggplot() +
        geom_line(data=p.rrho,  aes(x = n, y = time, color = "RedRibbon\nPlaisier et al."),  size = 1.2) +
        geom_line(data=p.rrho.orig, aes(x = n, y = time, color = "Plaisier et al."), size = 1.2) +  
        scale_x_continuous(breaks= pretty_breaks()) +
        xlab("Size of the lists") + ylab("Time (seconds)") +
        geom_point(data=p.rrho,  aes(x = n, y = time, color = "RedRibbon\nPlaisier et al."), size=3) +
        geom_point(data=p.rrho.orig, aes(x = n, y = time, color = "Plaisier et al."), size=3)  + labs(color = "") +
        ## ggtitle("Uncorrected pvalue") + theme(plot.title = "none") + ## element_text(size=title.size))+
        geom_vline(xintercept = 19962, linetype = 3) + geom_text(aes(x=19962 + 10000, label="protein coding genes", y=140), colour="#505050", angle=90) +
        geom_vline(xintercept = 60660, linetype = 3) + geom_text(aes(x=60660 + 10000, label="genes", y=170), colour="#505050", angle=90) +
        geom_vline(xintercept = 127802, linetype = 3) + geom_text(aes(x=127802 + 10000, label="lncRNA", y=170), colour="#505050", angle=90) + 
        geom_vline(xintercept = 232117, linetype = 3) + geom_text(aes(x=232117 + 10000, label="transcripts", y=170), colour="#505050", angle=90) + 
        theme(axis.text.x = element_text(angle = x.angle, vjust = 1, hjust=1)) +
        theme(text = element_text(size=text.size)) +
        theme(legend.position = "top", ,# c(0.7, 0.5),
              legend.background = element_rect(fill=alpha('white', 0.2)),
              legend.text = element_text(size=8),
              legend.title = element_text(size=10))
        
    
    
    
    p.rrho.permutation <- fread("input/p-rrho-permutation.tsv",sep="\t")
    p.rrho.permutation.ea <- fread("input/p-rrho-permutation-ea.tsv",sep="\t")

    gg_rrho_permutation <- ggplot() +
        geom_line(data=p.rrho.permutation[n.threads > 1], aes(x = n, y = time, color = "RedRibbon\nPlaisier et al."), size = 1.2) +
        geom_line(data=p.rrho.permutation.ea, aes(x = n, y = time, color = "RedRibbon EA"), size = 1.2) +
        ## scale_color_discrete(name = "Permutations", labels = c("EA", "Classic")) +
        scale_x_continuous(breaks= pretty_breaks()) +
        xlab("Size of the lists") + ylab("Time (seconds)") +
        geom_point(data=p.rrho.permutation[n.threads > 1], aes(x = n, y = time, color = "RedRibbon\nPlaisier et al."), size=3) +
        geom_point(data=p.rrho.permutation.ea, aes(x = n, y = time, color = "RedRibbon EA"), size=3) + labs(color = "") +
        ## ggtitle("Corrected pvalue") + theme(plot.title = "none") + ## element_text(size=title.size))+ 
        geom_vline(xintercept = 19962, linetype = 3) + geom_text(aes(x=19962 + 10000, label="protein coding genes", y=60), colour="#505050", angle=90) +
        geom_vline(xintercept = 60660, linetype = 3) + geom_text(aes(x=60660 + 10000, label="genes", y=70), colour="#505050", angle=90) +
        geom_vline(xintercept = 127802, linetype = 3) + geom_text(aes(x=127802 - 10000, label="lncRNA", y=70), colour="#505050", angle=90) + 
        geom_vline(xintercept = 232117, linetype = 3) + geom_text(aes(x=232117 + 10000, label="transcripts", y=70), colour="#505050", angle=90) + 
        theme(axis.text.x = element_text(angle = x.angle, vjust = 1, hjust=1))  + theme(text = element_text(size=text.size)) +
        theme(legend.position = "top",
              legend.background = element_rect(fill=alpha('white', 0.2)),
              legend.text = element_text(size=8),
              legend.title = element_text(size=10))


    ## original cpu https://www.cpubenchmark.net/cpu.php?cpu=Intel+Xeon+3.20GHz&id=1320 single thread rating = 696
    ## our server cpuhttps://www.cpubenchmark.net/cpu.php?cpu=Intel+Xeon+E5-2650+v4+%40+2.20GHz&id=2797  Single Thread Rating = 1201
    scale_factor <-  696 / 1201
    df <- data.frame(soft=c("Plaisier et al.", "RedRibbon", "// RedRibbon", "// RedRibbon EA" ),
                     time=c(4 * 3600 * scale_factor,
                            p.rrho.permutation[n.threads == 1]$time,
                            p.rrho.permutation[n.threads > 1 & step == 50]$time,
                            p.rrho.permutation.ea[n == 5000]$time))
    df$soft <- factor(df$soft, levels = df$soft)
    gg.cmp <- ggplot(data=df, aes(x=soft, y=time)) +
        geom_bar(stat="identity") +
        scale_y_continuous(trans = pseudo_log_trans(base = 10), breaks=c(0,10,100,1000,10000,100000)) +
        geom_text(data=df,aes(x=soft,y=time,label=formatC(time, format = "f", digits = 1)),vjust=0, size=3) +
        ylab("Time (seconds)") + xlab("Software") +
        theme(axis.text.x = element_text(angle = x.angle, vjust = 1, hjust=1),
              text = element_text(size=text.size)) ## +
        ## ggtitle("Corrected pvalue with classical algorithm\n (5000 genes, step size of 50)") + theme(plot.title = "none") ## element_text(size=title.size))

    return( list(combined = ggarrange(gg_rrho + coord_fixed(ratio=1),
                                      gg.cmp + coord_fixed(ratio=1) + ylab(""),
                                      gg_rrho_permutation + coord_fixed(ratio=1) + ylab(""),
                                      ## labels = c("A", "B", "C"),
                                      ncol = 3, nrow = 1),
                 redribbon.rrho = gg_rrho,
                 comparison = gg.cmp,
                 redribbon.perm = gg_rrho_permutation
                 ) )
}
