library(AnnotationDbi)
library(GenomicFeatures)
library(biomaRt)

cache.file <- function (fn)
{
    cache.dir <- file.path("~", ".cache", "redribbon")
    cache.fn <- file.path(cache.dir, paste0(basename(fn), ".sqlite"))
    dir.create(cache.dir, showWarnings=FALSE, recursive=T)
    
    return(cache.fn)
}

load_gtf <- function(fn = "input/gtf/gencode.v37.annotation.gtf.gz")
{
    cache.fn <- cache.file(fn)
    
    if (file.exists(cache.fn))
        txdb <- AnnotationDbi::loadDb(cache.fn)
    else
    {
        txdb <- GenomicFeatures::makeTxDbFromGFF(fn, format = "gtf")
        AnnotationDbi::saveDb(txdb, cache.fn)
    }
    return(txdb)
}

tx2gene <- function(fn = "input/gtf/gencode.v37.annotation.gtf.gz")
{
    txdb <- load_gtf(fn)
    k <- biomaRt::keys(txdb, keytype = "TXNAME")
    tx <- biomaRt::select(txdb, k,  "GENEID", "TXNAME")
    row.names(tx) <- tx$TXNAME
    
    return(tx)
}
